/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *****************************************************************************
 *
 *        Stargazer-a
 *
 * $Id: 00-base-00.sql,v 1.6 2008/05/11 17:05:53 faust Exp $
 *
 * .
 *      * dm_permission_flag.     - rw.
 *          r - , w -  .
 *          0 -  , 1 -  
 *
 *      * dm_traff_type.     :
 *          0 - up -   upload
 *          1 - down -   download
 *          2 - max -     upload/download
 *          3 - up+down -    upload  download
 *                 - 
 *         .         .
 *          "   ?"
 *
 *      * dm_ip. IP         .
 *               IP  !!!
 *
 *      * dm_period.     .
 *            (int16).    0 
 *             .
 *                
 *          .
 *
 *      * dm_session_event_type.       .
 *        'c' - connect, 'd' - disconnect.
 *
 *****************************************************************************
 */

/*
 *connect 'localhost:/var/stg/stargazer.fdb' user 'stg' password '123456';
 *drop database;
 *
 *create database 'localhost:/var/stg/stargazer.fdb' user 'stg' password '123456' default character set win1251;
 */
 

/*
 *****************************************************************************
 * -=   =-
 *****************************************************************************
 */

create domain dm_id as integer not null;
create domain dm_null_id as integer;
create domain dm_login as varchar(32) not null;
create domain dm_tariff_name as varchar(32) not null;
create domain dm_group_name as varchar(32);
create domain dm_corporation_name as varchar(32);
create domain dm_parameter_name as varchar(32);

create domain dm_password as varchar(64) not null;
/* bitmask - rw => Read, Write */
create domain dm_permission_flag as smallint not null
    check ( value between 0 and 3 );
create domain dm_money as numeric(10,6) not null;
/* (0, 1, 2, 3) => (up, down, max, up+down) */
create domain dm_traff_type as smallint not null
    check ( value between 0 and 3 );
create domain dm_dir_num as smallint not null;
create domain dm_num as smallint not null;
create domain dm_traffic_mb as integer not null;
create domain dm_traffic_byte as bigint not null;
create domain dm_time as time not null;
create domain dm_moment as timestamp not null;
create domain dm_credit_moment as timestamp;
create domain dm_ip as integer not null;
create domain dm_mask as integer not null;
create domain dm_user_address as varchar(256) default '';
create domain dm_bool as char(1) not null
    check ( value in ('0', '1', 't', 'f', 'T', 'F') );
create domain dm_email as varchar(256) default '';
create domain dm_note as varchar(256) default '';
create domain dm_phone as varchar(256) default '';
create domain dm_user_name as varchar(256) default '';
create domain dm_service_comment as varchar(256) default '';
create domain dm_service_name varchar(32) default '';
/* TODO: why 0-31? Which is default? */
create domain dm_pay_day as smallint not null
    check ( value between 0 and 31 );
create domain dm_period as integer not null;
create domain dm_counter as smallint not null;
/* Is it needded? */
create domain dm_message_ver as integer not null;
create domain dm_message_type as integer not null;
/*----------------*/
create domain dm_message as varchar(256) not null;
create domain dm_user_data as varchar(256) not null;
create domain dm_session_event_type as char(1) not null
    check ( value in ('c', 'd') );
create domain dm_char_value as varchar(64) not null;
create domain dm_date as date not null;



/*
 *****************************************************************************
 * -=   =-
 *****************************************************************************
 */

create table tb_admins
(
    pk_admin dm_id primary key,
    login dm_login unique,
    passwd dm_password,
    chg_conf dm_permission_flag,
    chg_password dm_permission_flag,
    chg_stat dm_permission_flag,
    chg_cash dm_permission_flag,
    usr_add_del dm_permission_flag,
    chg_tariff dm_permission_flag,
    chg_admin dm_permission_flag,
    chg_service dm_permission_flag,
    chg_corporation dm_permission_flag
);

create table tb_tariffs
(
    pk_tariff dm_id primary key,
    name dm_tariff_name unique,
    fee dm_money,
    free dm_money,
    passive_cost dm_money,
    traff_type dm_traff_type
);

create table tb_tariffs_params
(
    pk_tariff_param dm_id primary key,
    fk_tariff dm_id,
    dir_num dm_dir_num,
    price_day_a dm_money,
    price_day_b dm_money,
    price_night_a dm_money,
    price_night_b dm_money,
    threshold dm_traffic_mb,
    time_day_begins dm_time,
    time_day_ends dm_time,

    foreign key (fk_tariff) references tb_tariffs (pk_tariff)
);

create table tb_corporations
(
    pk_corporation dm_id primary key,
    name dm_corporation_name unique,
    cash dm_money
);

create table tb_users
(
    pk_user dm_id primary key,
    fk_tariff dm_null_id,
    fk_tariff_change dm_null_id,
    fk_corporation dm_null_id,
    address dm_user_address,
    always_online dm_bool,
    credit dm_money,
    credit_expire dm_credit_moment,
    disabled dm_bool,
    disabled_detail_stat dm_bool,
    email dm_email,
    grp dm_group_name,
    note dm_note,
    passive dm_bool,
    passwd dm_password,
    phone dm_phone,
    name dm_login unique,
    real_name dm_user_name,

    foreign key (fk_tariff) references tb_tariffs (pk_tariff),
    foreign key (fk_tariff_change) references tb_tariffs (pk_tariff),
    foreign key (fk_corporation) references tb_corporations (pk_corporation)
);

create table tb_detail_stats
(
    pk_detail_stat dm_id primary key,
    fk_user dm_id,
    dir_num dm_dir_num,
    ip dm_ip,
    download dm_traffic_byte,
    upload dm_traffic_byte,
    cost dm_money,
    from_time dm_moment,
    till_time dm_moment,

    foreign key (fk_user) references tb_users (pk_user)
);

create table tb_services
(
    pk_service dm_id primary key,
    name dm_service_name unique,
    comment dm_service_comment,
    cost dm_money,
    pay_day dm_pay_day
);

create table tb_users_services
(
    pk_user_service dm_id primary key,
    fk_user dm_id,
    fk_service dm_id,

    foreign key (fk_user) references tb_users (pk_user),
    foreign key (fk_service) references tb_services (pk_service)
);

create table tb_messages
(
    pk_message dm_id primary key,
    fk_user dm_id,
    ver dm_message_ver,
    msg_type dm_message_type,
    last_send_time dm_period,
    creation_time dm_period,
    show_time dm_period,
    repeat dm_counter,
    repeat_period dm_period,
    msg_text dm_message,

    foreign key (fk_user) references tb_users (pk_user)
);

create table tb_stats
(
    pk_stat dm_id primary key,
    fk_user dm_id,
    cash dm_money,
    free_mb dm_money,
    last_activity_time dm_moment,
    last_cash_add dm_money,
    last_cash_add_time dm_moment,
    passive_time dm_period,
    stats_date dm_date,

    foreign key (fk_user) references tb_users (pk_user)
);

create table tb_stats_traffic
(
    pk_stat_traffic dm_id primary key,
    fk_stat dm_id,
    dir_num dm_dir_num,
    download dm_traffic_byte,
    upload dm_traffic_byte,

    foreign key (fk_stat) references tb_stats (pk_stat)
);

create table tb_users_data
(
    pk_user_data dm_id primary key,
    fk_user dm_id,
    num dm_num, /* data_id dm_id renamed */
    data dm_user_data,

    foreign key (fk_user) references tb_users (pk_user)
);

create table tb_allowed_ip
(
    pk_allowed_ip dm_id primary key,
    fk_user dm_id,
    ip dm_ip,
    mask dm_mask,

    foreign key (fk_user) references tb_users (pk_user)
);

create table tb_sessions_log
(
    pk_session_log dm_id primary key,
    fk_user dm_id,
    event_time dm_moment,
    event_type dm_session_event_type,
    ip dm_ip,

    foreign key (fk_user) references tb_users (pk_user)
);

create table tb_sessions_data
(
    pk_session_data dm_id primary key,
    fk_session_log dm_id,
    dir_num dm_dir_num,
    session_upload dm_traffic_byte,
    session_download dm_traffic_byte,
    month_upload dm_traffic_byte,
    month_download dm_traffic_byte,

    foreign key (fk_session_log) references tb_sessions_log (pk_session_log)
);

create table tb_parameters
(
    pk_parameter dm_id primary key,
    name dm_parameter_name unique
);

create table tb_params_log
(
    pk_param_log dm_id primary key,
    fk_user dm_id,
    fk_parameter dm_id,
    event_time dm_moment,
    from_val dm_char_value,
    to_val dm_char_value,
    comment dm_service_comment,

    foreign key (fk_user) references tb_users (pk_user),
    foreign key (fk_parameter) references tb_parameters (pk_parameter)
);


/*
 *****************************************************************************
 * -=   =-
 *****************************************************************************
 */



/*
 *****************************************************************************
 * -=   =-
 *****************************************************************************
 */

create  generator gn_pk_admin;
set     generator gn_pk_admin           to 0;
create  generator gn_pk_tariff;
set     generator gn_pk_tariff          to 0;
create  generator gn_pk_tariff_param;
set     generator gn_pk_tariff_param    to 0;
create  generator gn_pk_corporation;
set     generator gn_pk_corporation     to 0;
create  generator gn_pk_user;
set     generator gn_pk_user            to 0;
create  generator gn_pk_detail_stat;
set     generator gn_pk_detail_stat     to 0;
create  generator gn_pk_service;
set     generator gn_pk_service         to 0;
create  generator gn_pk_user_service;
set     generator gn_pk_user_service    to 0;
create  generator gn_pk_message;
set     generator gn_pk_message         to 0;
create  generator gn_pk_stat;
set     generator gn_pk_stat            to 0;
create  generator gn_pk_stat_traffic;
set     generator gn_pk_stat_traffic    to 0;
create  generator gn_pk_user_data;
set     generator gn_pk_user_data       to 0;
create  generator gn_pk_allowed_ip;
set     generator gn_pk_allowed_ip      to 0;
create  generator gn_pk_session;
set     generator gn_pk_session         to 0;
create  generator gn_pk_session_log;
set     generator gn_pk_session_log     to 0;
create  generator gn_pk_session_data;
set     generator gn_pk_session_data    to 0;
create  generator gn_pk_parameter;
set     generator gn_pk_parameter       to 0;
create  generator gn_pk_param_log;
set     generator gn_pk_param_log       to 0;


/*
 *****************************************************************************
 * -=   =-
 *****************************************************************************
 */

set term !! ;
create trigger tr_admin_bi for tb_admins
active before insert position 0
as
begin
    if (new.pk_admin is null)
    then new.pk_admin = gen_id(gn_pk_admin, 1);
end !!
set term ; !!

/*set term !! ;
create trigger tr_tariff_bi for tb_tariffs active
before insert position 0
as
begin
    if (new.pk_tariff is null)
    then new.pk_tariff = gen_id(gn_pk_tariff, 1);
end !!
set term ; !!*/

set term !! ;
create trigger tr_tariff_param_bi for tb_tariffs_params active
before insert position 0
as
begin
    if (new.pk_tariff_param is null)
    then new.pk_tariff_param = gen_id(gn_pk_tariff_param, 1);
end !!
set term ; !!

set term !! ;
create trigger tr_corporation_bi for tb_corporations active
before insert position 0
as
begin
    if (new.pk_corporation is null)
    then new.pk_corporation = gen_id(gn_pk_corporation, 1);
end !!
set term ; !!

/*set term !! ;
create trigger tr_user_bi for tb_users active
before insert position 0
as
begin
    if (new.pk_user is null)
    then new.pk_user = gen_id(gn_pk_user, 1);
end !!
set term ; !!*/

set term !! ;
create trigger tr_detail_stat_bi for tb_detail_stats active
before insert position 0
as
begin
    if (new.pk_detail_stat is null)
    then new.pk_detail_stat = gen_id(gn_pk_detail_stat, 1);
end !!
set term ; !!

set term !! ;
create trigger tr_service_bi for tb_services active
before insert position 0
as
begin
    if (new.pk_service is null)
    then new.pk_service = gen_id(gn_pk_service, 1);
end !!
set term ; !!

set term !! ;
create trigger tr_user_service_bi for tb_users_services active
before insert position 0
as
begin
    if (new.pk_user_service is null)
    then new.pk_user_service = gen_id(gn_pk_user_service, 1);
end !!
set term ; !!

/*set term !! ;
create trigger tr_message_bi for tb_messages active
before insert position 0
as
begin
    if (new.pk_message is null)
    then new.pk_message = gen_id(gn_pk_message, 1);
end !!
set term ; !!*/

/*set term !! ;
create trigger tr_stat_bi for tb_stats active
before insert position 0
as
begin
    if (new.pk_stat is null)
    then new.pk_stat = gen_id(gn_pk_stat, 1);
end !!
set term ; !!*/

set term !! ;
create trigger tr_stat_traffic_bi for tb_stats_traffic active
before insert position 0
as
begin
    if (new.pk_stat_traffic is null)
    then new.pk_stat_traffic = gen_id(gn_pk_stat_traffic, 1);
end !!
set term ; !!

set term !! ;
create trigger tr_user_data_bi for tb_users_data active
before insert position 0
as
begin
    if (new.pk_user_data is null)
    then new.pk_user_data = gen_id(gn_pk_user_data, 1);
end !!
set term ; !!

set term !! ;
create trigger tr_allowed_ip_bi for tb_allowed_ip active
before insert position 0
as
begin
    if (new.pk_allowed_ip is null)
    then new.pk_allowed_ip = gen_id(gn_pk_allowed_ip, 1);
end !!
set term ; !!

/*set term !! ;
create trigger tr_session_log_bi for tb_sessions_log active
before insert position 0
as
begin
    if (new.pk_session_log is null)
    then new.pk_session_log = gen_id(gn_pk_session_log, 1);
end !!
set term ; !!*/

set term !! ;
create trigger tr_session_data_bi for tb_sessions_data active
before insert position 0
as
begin
    if (new.pk_session_data is null)
    then new.pk_session_data = gen_id(gn_pk_session_data, 1);
end !!
set term ; !!

set term !! ;
create trigger tr_parameter_bi for tb_parameters active
before insert position 0
as
begin
    if (new.pk_parameter is null)
    then new.pk_parameter = gen_id(gn_pk_parameter, 1);
end !!
set term ; !!

set term !! ;
create trigger tr_param_log_bi for tb_params_log active
before insert position 0
as
begin
    if (new.pk_param_log is null)
    then new.pk_param_log = gen_id(gn_pk_param_log, 1);
end !!
set term ; !!

/*
 *****************************************************************************
 * -=  stored procedure =-
 *****************************************************************************
 */

/*
 * Add a message returning it's ID
 */
set term !! ;
create procedure sp_add_message(pk_message integer, login varchar(32), ver integer, msg_type integer, last_send_time integer, creation_time integer, show_time integer, repeat integer, repeat_period integer, msg_text varchar(256))
returns(res integer)
as  
begin
    if (:pk_message is null) then
    begin
	pk_message = gen_id(gn_pk_message, 1);
	insert into tb_messages values (:pk_message,
					(select pk_user from tb_users where name = :login),
					:ver,
					:msg_type,
					:last_send_time,
					:creation_time, 
					:show_time,
					:repeat,
					:repeat_period,
					:msg_text);
    end
    else
    begin
	update tb_messages set fk_user = (select pk_user from tb_users where name = :login),
			       ver = :ver,
			       msg_type = :msg_type,
			       last_send_time = :last_send_time,
			       creation_time = :creation_time,
			       show_time = :show_time,
			       repeat = :repeat_period,
			       repeat_period = :repeat_period,
			       msg_text = :msg_text
			   where pk_message = :pk_message;
    end
    res = :pk_message;
end !!
set term ; !!

set term !! ;
create procedure sp_delete_service(name varchar(32))
as
declare variable pk_service integer;
begin
    select pk_service from tb_services where name = :name into pk_service;
    if (pk_service is  not null) then
    begin
	delete from tb_users_services where fk_service = :pk_service;
	delete from tb_services where pk_service = :pk_service;
    end
end !!
set term ; !!

set term !! ;
create procedure sp_add_tariff(name varchar(32), dirs integer)
as
declare variable pk_tariff integer;
begin
    pk_tariff = gen_id(gn_pk_tariff, 1);
    insert into tb_tariffs (pk_tariff, name, fee, free, passive_cost, traff_type) values (:pk_tariff, :name, 0, 0, 0, 0);
    while (dirs > 0) do
    begin
        insert into tb_tariffs_params (fk_tariff, dir_num, price_day_a, 
					price_day_b, price_night_a, price_night_b, 
					threshold, time_day_begins, time_day_ends)
		    values (:pk_tariff, :dirs - 1, 0, 0, 0, 0, 0, '0:0', '0:0');
	dirs = dirs - 1;
    end
end !!
set term ; !!

set term !! ;
create procedure sp_delete_tariff(name varchar(32))
as
declare variable pk_tariff integer;
begin
    select pk_tariff from tb_tariffs where name = :name into pk_tariff;
    if (pk_tariff is not null) then
    begin
	delete from tb_tariffs_params where fk_tariff = :pk_tariff;
	delete from tb_tariffs where pk_tariff = :pk_tariff;
    end
end !!
set term ; !!

set term !! ;
create procedure sp_add_user(name varchar(32), dirs integer)
as
declare variable pk_user integer;
declare variable pk_stat integer;
begin
    pk_user = gen_id(gn_pk_user, 1);
    insert into tb_users(pk_user, fk_tariff, fk_tariff_change, fk_corporation, address, always_online, credit, credit_expire, disabled, disabled_detail_stat, email, grp, note, passive, passwd, phone, name, real_name) values (:pk_user, NULL, NULL, NULL, '', 0, 0, 'now', 0, 0, '', '_', '', 0, '', '', :name, '');
    pk_stat = gen_id(gn_pk_stat, 1);
    insert into tb_stats values (:pk_stat, :pk_user, 0, 0, 'now', 0, 'now', 0, 'now');
    while (dirs > 0) do
    begin
	insert into tb_stats_traffic (fk_stat, dir_num, upload, download) values (:pk_stat, :dirs - 1, 0, 0);
	dirs = dirs - 1;
    end
end!!
set term ; !!

set term !! ;
create procedure sp_delete_user(name varchar(32))
as
declare variable pk_user integer;
begin
    select pk_user from tb_users where name = :name into pk_user;
    if (pk_user is not null) then
    begin
	delete from tb_users_services where fk_user = :pk_user;
	delete from tb_params_log where fk_user = :pk_user;
	delete from tb_detail_stats where fk_user = :pk_user;
	delete from tb_stats_traffic where fk_stat in (select pk_stat from tb_stats where fk_user = :pk_user);
	delete from tb_stats where fk_user = :pk_user;
	delete from tb_sessions_data where fk_session_log in (select pk_session_log from tb_sessions_log where fk_user = :pk_user);
	delete from tb_sessions_log where fk_user = :pk_user;
	delete from tb_allowed_ip where fk_user = :pk_user;
	delete from tb_users_data where fk_user = :pk_user;
	delete from tb_messages where fk_user = :pk_user;
	delete from tb_users where pk_user = :pk_user;
    end
end !!
set term ; !!

set term !! ;
create procedure sp_append_session_log(name varchar(32), event_time timestamp, event_type char(1), ip integer)
returns(pk_session_log integer)
as
begin
    pk_session_log = gen_id(gn_pk_session_log, 1);
    insert into tb_sessions_log (pk_session_log, fk_user, event_time, event_type, ip) values (:pk_session_log, (select pk_user from tb_users where name = :name), :event_time, :event_type, :ip);
end !!
set term ; !!

set term !! ;
create procedure sp_add_stat(name varchar(32), cash numeric(10,6), free_mb numeric(10,6), last_activity_time timestamp, last_cash_add numeric(10,6), last_cash_add_time timestamp, passive_time integer, stats_date date)
returns(pk_stat integer)
as
begin
    pk_stat = gen_id(gn_pk_stat, 1);
    insert into tb_stats (pk_stat, fk_user, cash, free_mb, last_activity_time, last_cash_add, last_cash_add_time, passive_time, stats_date) values (:pk_stat, (select pk_user from tb_users where name = :name), :cash, :free_mb, :last_activity_time, :last_cash_add, :last_cash_add_time, :passive_time, :stats_date);
end !!
set term ; !!

/*
 *****************************************************************************
 * -=   =-
 *****************************************************************************
 */

insert into tb_admins values(0, 'admin', 'geahonjehjfofnhammefahbbbfbmpkmkmmefahbbbfbmpkmkmmefahbbbfbmpkmk', 1, 1, 1, 1, 1, 1, 1, 1, 1);

/* EOF */

